@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # Copyright (c) 2019 Jordi Romera <jordiromera@users.sourceforge.net>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import GLib, GObject, Gio
+from gi.repository import GLib, GObject, Gio, Gtk
 
 from gi.repository.Gio import FILE_ATTRIBUTE_STANDARD_NAME, \
                               FILE_ATTRIBUTE_STANDARD_TYPE, \
@@ -113,7 +113,7 @@ class CollectionScanner(GObject.GObject, TagReader):
             # Launch scan in a separate thread
             self.__thread = App().task_helper.run(self.__scan, scan_type, uris)
 
-    def save_album(self, item,):
+    def save_album(self, item):
         """
             Add album to DB
             @param item as CollectionItem
@@ -189,8 +189,8 @@ class CollectionScanner(GObject.GObject, TagReader):
                                          item.discnumber,
                                          item.discname,
                                          item.album_id,
-                                         item.year,
-                                         item.timestamp,
+                                         item.original_year,
+                                         item.original_timestamp,
                                          item.track_pop,
                                          item.track_rate,
                                          item.track_loved,
@@ -232,18 +232,12 @@ class CollectionScanner(GObject.GObject, TagReader):
         lp_album_id = get_lollypop_album_id(item.album_name,
                                             item.album_artists)
         if lp_album_id != item.lp_album_id:
-            App().art.move_artwork(item.lp_album_id, lp_album_id)
+            App().album_art.move(item.lp_album_id, lp_album_id)
             App().albums.set_lp_album_id(item.album_id, lp_album_id)
             item.lp_album_id = lp_album_id
         # Update album genres
         for genre_id in item.genre_ids:
             App().albums.add_genre(item.album_id, genre_id)
-        # Update year based on tracks
-        year = App().tracks.get_year_for_album(item.album_id)
-        if year is not None:
-            App().albums.set_year(item.album_id, year)
-            timestamp = App().tracks.get_timestamp_for_album(item.album_id)
-            App().albums.set_timestamp(item.album_id, timestamp)
         App().cache.clear_durations(item.album_id)
 
     def update_track(self, item):
@@ -337,7 +331,7 @@ class CollectionScanner(GObject.GObject, TagReader):
         App().window.container.progress.add(self)
         App().window.container.progress.set_fraction(0, self)
         self.__progress_fraction = 0
-        notification = AppNotification(_("Resetting database"), [], [])
+        notification = AppNotification(_("Resetting database"), [], [], 10000)
         notification.show()
         App().window.container.add_overlay(notification)
         notification.set_reveal_child(True)
@@ -506,6 +500,7 @@ class CollectionScanner(GObject.GObject, TagReader):
             (files, dirs, streams) = self.__get_objects_for_uris(
                 scan_type, uris)
             if len(uris) != len(streams) and not files:
+                self.__flatpak_migration()
                 App().notify.send("Lollypop",
                                   _("Scan disabled, missing collection"))
                 return
@@ -562,6 +557,8 @@ class CollectionScanner(GObject.GObject, TagReader):
         except Exception as e:
             Logger.warning("CollectionScanner::__scan(): %s", e)
         SqlCursor.remove(App().db)
+        App().settings.set_value("flatpak-access-migration",
+                                 GLib.Variant("b", True))
 
     def __scan_to_handle(self, uri):
         """
@@ -775,17 +772,20 @@ class CollectionScanner(GObject.GObject, TagReader):
         discnumber = self.get_discnumber(tags)
         discname = self.get_discname(tags)
         tracknumber = self.get_tracknumber(tags, name)
-        if track_rate == 0:
-            track_rate = self.get_popm(tags)
+        # We have popm in tags, override history one
+        tag_track_rate = self.get_popm(tags)
+        if tag_track_rate > 0:
+            track_rate = tag_track_rate
         if album_mtime == 0:
             album_mtime = track_mtime
         bpm = self.get_bpm(tags)
         compilation = self.get_compilation(tags)
-        year = None
-        if not App().settings.get_value("ignore-original-date"):
-            (year, timestamp) = self.get_original_year(tags)
+        (original_year, original_timestamp) = self.get_original_year(tags)
+        (year, timestamp) = self.get_year(tags)
         if year is None:
-            (year, timestamp) = self.get_year(tags)
+            (year, timestamp) = (original_year, original_timestamp)
+        elif original_year is None:
+            (original_year, original_timestamp) = (year, timestamp)
         # If no artists tag, use album artist
         if artists == "":
             artists = album_artists
@@ -803,14 +803,16 @@ class CollectionScanner(GObject.GObject, TagReader):
         return (title, artists, genres, a_sortnames, aa_sortnames,
                 album_artists, album_name, discname, album_loved, album_mtime,
                 album_synced, album_rate, album_pop, discnumber, year,
-                timestamp, mb_album_id, mb_track_id, mb_artist_id,
+                timestamp, original_year, original_timestamp,
+                mb_album_id, mb_track_id, mb_artist_id,
                 mb_album_artist_id, tracknumber, track_pop, track_rate, bpm,
                 track_mtime, track_ltime, track_loved, duration, compilation)
 
     def __add2db(self, uri, name, artists,
                  genres, a_sortnames, aa_sortnames, album_artists, album_name,
                  discname, album_loved, album_mtime, album_synced, album_rate,
-                 album_pop, discnumber, year, timestamp, mb_album_id,
+                 album_pop, discnumber, year, timestamp,
+                 original_year, original_timestamp, mb_album_id,
                  mb_track_id, mb_artist_id, mb_album_artist_id,
                  tracknumber, track_pop, track_rate, bpm, track_mtime,
                  track_ltime, track_loved, duration, compilation,
@@ -839,6 +841,8 @@ class CollectionScanner(GObject.GObject, TagReader):
                               discnumber=discnumber,
                               year=year,
                               timestamp=timestamp,
+                              original_year=original_year,
+                              original_timestamp=original_timestamp,
                               mb_album_id=mb_album_id,
                               mb_track_id=mb_track_id,
                               mb_artist_id=mb_artist_id,
@@ -856,3 +860,15 @@ class CollectionScanner(GObject.GObject, TagReader):
         self.save_album(item)
         self.save_track(item)
         return item
+
+    def __flatpak_migration(self):
+        """
+            https://github.com/flathub/org.gnome.Lollypop/pull/108
+        """
+        if GLib.file_test("/app", GLib.FileTest.EXISTS) and\
+                not App().settings.get_value("flatpak-access-migration"):
+            from lollypop.assistant_flatpak import FlatpakAssistant
+            assistant = FlatpakAssistant()
+            assistant.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
+            assistant.set_transient_for(App().window)
+            GLib.timeout_add(1000, assistant.show)
